﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.IO;
using System.Net;


namespace Port_Scanner {
    public partial class Form1 : Form {
        Port_Scanner portScanner;
        delegate void PortAdderDelegate(Object o);
        bool scanning = false;
        Int32 portsToCheck = 0;
        Int32 portsChecked = 0;
        object syncObject = new object();
        public Form1() {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e) {
            if (!scanning) {
                UInt16 minPortRange, maxPortRange = 1024;

                bool parse1 = !UInt16.TryParse(textBox2.Text, out minPortRange);
                bool parse2 = !UInt16.TryParse(textBox3.Text, out maxPortRange);

                if (parse1 || parse2) {
                    MessageBox.Show(String.Format("The entered port range is not valid.\r\nThe port range should be between {0}-{1}.", UInt16.MinValue, UInt16.MaxValue), "Invalid port range", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }


                listBox1.Items.Clear();
                listBox2.Items.Clear();

                button1.Text = "Abort";
                Status("scanning...");
                textBox1.Enabled = false;
                textBox2.Enabled = false;
                textBox3.Enabled = false;
                scanning = true;

                portsChecked = 0;
                portsToCheck = maxPortRange - minPortRange;

                portScanner = new Port_Scanner(minPortRange, maxPortRange, getIP(resolveIP(textBox1.Text)));

                portScanner.PortCheckCompleted += new EventHandler<PortEventArgs>(portChecker_PortCheckCompleted);
                portScanner.StartChecking();

            } else {
                if (portScanner != null) {
                    portScanner.End();
                }
                Status("aborting...");
                CloseHandler();
                button1.Text = "Scan";
                textBox1.Enabled = true;
                textBox2.Enabled = true;
                textBox3.Enabled = true;
                scanning = false;
            }
        }
        private String resolveIP(String address) {
            String webAddress = address;
            webAddress = webAddress.Replace("https://", "");
            webAddress = webAddress.Replace("http://", "");
            webAddress = webAddress.Split(Char.Parse(@"/"))[0];

            try {
                address = Dns.GetHostEntry(webAddress).AddressList[0].ToString();

            } catch (Exception) {
                address = webAddress;
            }

            return address;
        }
        private IPAddress getIP(String address) {
            IPAddress ip = IPAddress.Loopback;

            if (IPAddress.TryParse(address, out ip)) {
                return ip;
            } else {
                return IPAddress.Loopback;

            }
        }

        private void Form1_Load(object sender, EventArgs e) {
            Status("inactive.");
        }

        public void Status(String status) {
            label6.Invoke(new MethodInvoker(() => { label6.Text = "Status: " + status; }));
        }

        void AddOpenPort(Object obj) {
            if (listBox1.InvokeRequired)
                listBox1.Invoke(new PortAdderDelegate(AddOpenPort), obj);
            else {
                listBox1.Items.Add(obj);

                listBox1.SelectedIndex = listBox1.Items.Count - 1;
            }
        }

        void AddClosedPort(Object obj) {
            if (listBox2.InvokeRequired)
                listBox2.Invoke(new PortAdderDelegate(AddClosedPort), obj);
            else {
                listBox2.Items.Add(obj);

                listBox2.SelectedIndex = listBox2.Items.Count - 1;
            }
        }

        private void button2_Click(object sender, EventArgs e) {
            textBox1.Text = resolveIP(textBox1.Text);
        }
        private void CloseHandler() {
            if (portScanner != null) {
                bool wasRunning = portScanner._running;

                portScanner.End();

                if (wasRunning) {
                    Status("inactive.");
                }
            }
        }

        void portChecker_PortCheckCompleted(object sender, PortEventArgs e) {
            lock (syncObject)
                portsChecked += 1;

            if (e.Open) {
                AddOpenPort(String.Format("Port {0}: {1}", e.Port, e.Message));
            } else {
                AddClosedPort(String.Format("Port {0}: {1}", e.Port, e.Message));
            }
            if (portsChecked >= portsToCheck) {
                button1.Invoke(new MethodInvoker(() => { button1.Text = "Scan"; }));
                textBox1.Invoke(new MethodInvoker(() => { textBox1.Enabled = true; }));
                textBox2.Invoke(new MethodInvoker(() => { textBox2.Enabled = true; }));
                textBox3.Invoke(new MethodInvoker(() => { textBox3.Enabled = true; }));
                scanning = false;
                Status("inactive.");
            }
        }

        private void label7_Click(object sender, EventArgs e) {
            System.Diagnostics.Process.Start("http://www.blackhost.xyz");
        }
    }
}